@@ -8,8 +8,12 @@
 ##############################################################################
 
 import logging
+from collections import namedtuple
 from httplib import BAD_REQUEST, UNAUTHORIZED, FORBIDDEN, OK
+import shlex
+from cStringIO import StringIO
 
+from twisted.internet import reactor
 from twisted.internet.defer import (
     inlineCallbacks,
     returnValue,
@@ -26,7 +30,6 @@ except ImportError:
         pass
 
 from . import constants as c
-from kerberos import GSSError
 from .util import (
     _authenticate_with_kerberos,
     _get_agent,
@@ -43,7 +46,7 @@ from .util import (
     RequestError,
     _ErrorReader,
     _StringProtocol,
-    ET
+    ET,
 )
 from .shell import (
     _find_shell_id,
@@ -61,14 +64,55 @@ from .enumerate import (
     _MAX_REQUESTS_PER_ENUMERATION
 )
 from .SessionManager import SESSION_MANAGER, Session
+kerberos = None
 LOG = logging.getLogger('winrm')
 
+EnumInfo = namedtuple('EnumInfo', ['wql', 'resource_uri'])
+
+
+def _build_ps_command_line_elem(ps_command, ps_script):
+    """Build PowerShell command line elements without splitting
+    the actual ps script into arguments. using _build_command_line_elem
+    with a ps script splits the script into separate arguments.  Remote
+    Windows shell inserts spaces when reconstituting the script.
+
+    ps_command - powershell command with arguments as string
+        e.g. 'powershell -NoLogo -NonInteractive -NoProfile -Command'
+    ps_script - script to be run in powershell as single line string
+        e.g. "& {get-counter -counter \"\memory\pages output/sec\" }"
+    """
+    command_line_parts = shlex.split(ps_command, posix=False)
+    # ensure '-command' is last
+    if command_line_parts[-1:][0].lower() != '-command':
+        index = 0
+        for option in command_line_parts:
+            if option.lower() == '-command':
+                command_line_parts.pop(index)
+                break
+            index += 1
+        command_line_parts.append(option)
+    prefix = "rsp"
+    ET.register_namespace(prefix, c.XML_NS_MSRSP)
+    command_line_elem = ET.Element('{%s}CommandLine' % c.XML_NS_MSRSP)
+    command_elem = ET.Element('{%s}Command' % c.XML_NS_MSRSP)
+    command_elem.text = command_line_parts[0]
+    command_line_elem.append(command_elem)
+    for arguments_text in command_line_parts[1:]:
+        arguments_elem = ET.Element('{%s}Arguments' % c.XML_NS_MSRSP)
+        arguments_elem.text = arguments_text
+        command_line_elem.append(arguments_elem)
+    arguments_elem = ET.Element('{%s}Arguments' % c.XML_NS_MSRSP)
+    arguments_elem.text = ps_script
+    command_line_elem.append(arguments_elem)
+    tree = ET.ElementTree(command_line_elem)
+    str_io = StringIO()
+    tree.write(str_io, encoding='utf-8')
+    return str_io.getvalue()
+
 
 class WinRMSession(Session):
     '''
     Session class to keep track of single winrm connection
-
-
     '''
     def __init__(self):
         super(WinRMSession, self).__init__()
@@ -93,6 +137,8 @@ class WinRMSession(Session):
         # connection.
         self.sem = DeferredSemaphore(1)
 
+        self._refresh_dc = None
+
     def is_kerberos(self):
         return self._conn_info.auth_type == 'kerberos'
 
@@ -161,12 +207,14 @@ class WinRMSession(Session):
                     raise e
             if response.code == UNAUTHORIZED:
                 if self.is_kerberos():
+                    if not kerberos:
+                        from .util import kerberos
                     auth_header = response.headers.getRawHeaders('WWW-Authenticate')[0]
                     auth_details = get_auth_details(auth_header)
                     try:
                         if auth_details:
                             self._gssclient._step(auth_details)
-                    except GSSError as e:
+                    except kerberos.GSSError as e:
                         msg = "HTTP Unauthorized received.  "
                         "Kerberos error code {0}: {1}.".format(e.args[1][1], e.args[1][0])
                         raise Exception(msg)
@@ -187,9 +235,9 @@ class WinRMSession(Session):
         returnValue(response)
 
     @inlineCallbacks
-    def send_request(self, request_template_name, client, **kwargs):
+    def send_request(self, request_template_name, client, envelope_size=None, **kwargs):
         response = yield self._send_request(
-            request_template_name, client, **kwargs)
+            request_template_name, client, envelope_size=envelope_size, **kwargs)
         proto = _StringProtocol()
         response.deliverBody(proto)
         body = yield proto.d
@@ -207,7 +255,11 @@ class WinRMSession(Session):
         returnValue(ET.fromstring(xml_str))
 
     @inlineCallbacks
-    def _send_request(self, request_template_name, client, **kwargs):
+    def _send_request(self, request_template_name, client, envelope_size=None,
+                      locale=None, code_page=None, **kwargs):
+        kwargs['envelope_size'] = envelope_size or client._conn_info.envelope_size
+        kwargs['locale'] = locale or client._conn_info.locale
+        kwargs['code_page'] = code_page or client._conn_info.code_page
         if self._login_d and not self._login_d.called:
             # check for a reconnection attempt so we do not send any requests
             # to a dead connection
@@ -233,14 +285,29 @@ class WinRMSession(Session):
         response = yield self.handle_response(request, response, client)
         returnValue(response)
 
+    def close_connection(self, client):
+        try:
+            self._refresh_dc.cancel()
+        except Exception:
+            pass
+
+        # Close the connection after 60 seconds.  This will give other clients
+        # enough time to keep the connection alive and continue using the same session.
+        self._refresh_dc = reactor.callLater(60, SESSION_MANAGER.close_connection, client)
+
 
 class WinRMClient(object):
+    """Base winrm client class
+
+    Contains core functionality for various types of winrm based clients
+    """
     def __init__(self, conn_info):
         verify_conn_info(conn_info)
         self.key = None
         self._conn_info = conn_info
         self.session_manager = SESSION_MANAGER
         self._session = None
+        self.ps_script = None
 
     @inlineCallbacks
     def init_connection(self):
@@ -285,7 +352,14 @@ class WinRMClient(object):
 
     @inlineCallbacks
     def _send_command(self, shell_id, command_line):
-        command_line_elem = _build_command_line_elem(command_line)
+        if self.ps_script is not None:
+            command_line_elem = _build_ps_command_line_elem(command_line,
+                                                            self.ps_script)
+        else:
+            command_line_elem = _build_command_line_elem(command_line)
+        LOG.debug('WinRMClient._send_command: sending command request '
+                  '(shell_id={0}, command_line_elem={1})'.format(
+                      shell_id, command_line_elem))
         command_elem = yield self._session.send_request(
             'command', self, shell_id=shell_id, command_line_elem=command_line_elem,
             timeout=self._conn_info.timeout)
@@ -299,28 +373,35 @@ class WinRMClient(object):
 
     @inlineCallbacks
     def close_connection(self):
-        yield self.session_manager.close_connection(self)
+        yield self._session.close_connection(self)
         returnValue(None)
 
 
 class SingleCommandClient(WinRMClient):
-
+    """Client to send a single command to a winrm device"""
     def __init__(self, conn_info):
         super(SingleCommandClient, self).__init__(conn_info)
         self.key = (self._conn_info.ipaddress, 'short')
 
     @inlineCallbacks
-    def run_command(self, command_line):
-        '''
-        Run a single command in the session's semaphore.  Windows must finish
+    def run_command(self, command_line, ps_script=None):
+        """Run a single command in the session's semaphore.  Windows must finish
         a command conversation before a new command or enumeration can start
-        '''
+
+        If running a powershell script, send it in separately with ps_script in
+        "& {<actual script here>}" format
+        e.g. command_line='powershell -NoLogo -NonInteractive -NoProfile -Command',
+        ps_script='"& {get-counter -counter \\\"\memory\pages output/sec\\\" }"'
+        """
+        cmd_response = None
+        self.ps_script = ps_script
         yield self.init_connection()
         try:
             cmd_response = yield self._session.sem.run(self.run_single_command,
                                                        command_line)
         except Exception:
             yield self.close_connection()
+            raise
         returnValue(cmd_response)
 
     @inlineCallbacks
@@ -335,7 +416,7 @@ class SingleCommandClient(WinRMClient):
                 .exit_code = <int>
         """
         shell_id = yield self._create_shell()
-
+        cmd_response = None
         try:
             cmd_response = yield self._run_command(shell_id, command_line)
         except TimeoutError:
@@ -368,6 +449,7 @@ class SingleCommandClient(WinRMClient):
 
 
 class LongCommandClient(WinRMClient):
+    """Client to run a single long running command to a winrm device"""
     def __init__(self, conn_info):
         super(LongCommandClient, self).__init__(conn_info)
         self._shell_id = None
@@ -375,15 +457,20 @@ class LongCommandClient(WinRMClient):
         self._exit_code = None
 
     @inlineCallbacks
-    def start(self, command_line):
+    def start(self, command_line, ps_script=''):
+        """Start long running command
+
+        If running a powershell script, send it in separately with ps_script in
+        "& {<actual script here>}" format
+        e.g. command_line='powershell -NoLogo -NonInteractive -NoProfile -Command',
+        ps_script='"& {get-counter -counter \\\"\memory\pages output/sec\\\" }"'
+
+        """
         LOG.debug("LongRunningCommand run_command: {0}".format(command_line))
-        self.key = (self._conn_info.ipaddress, command_line)
+        self.key = (self._conn_info.ipaddress, command_line + str(ps_script))
+        self.ps_script = ps_script
         yield self.init_connection()
         self._shell_id = yield self._create_shell()
-        command_line_elem = _build_command_line_elem(command_line)
-        LOG.debug('LongRunningCommand run_command: sending command request '
-                  '(shell_id={0}, command_line_elem={1})'.format(
-                      self._shell_id, command_line_elem))
         try:
             command_elem = yield self._send_command(self._shell_id,
                                                     command_line)
@@ -422,7 +509,8 @@ class LongCommandClient(WinRMClient):
 
 
 class EnumerateClient(WinRMClient):
-    """
+    """Client to send a single wmi query(WQL) to a winrm device
+
     Sends enumerate requests to a host running the WinRM service and returns
     a list of items.
     """
@@ -435,10 +523,8 @@ class EnumerateClient(WinRMClient):
 
     @inlineCallbacks
     def enumerate(self, wql, resource_uri=DEFAULT_RESOURCE_URI):
-        """
-        Runs a remote WQL query.
-        """
-        self.init_connection()
+        """Runs a remote WQL query."""
+        yield self.init_connection()
         request_template_name = 'enumerate'
         enumeration_context = None
         items = []
@@ -473,10 +559,9 @@ class EnumerateClient(WinRMClient):
 
     @inlineCallbacks
     def do_collect(self, enum_infos):
-        '''
-        Run enumerations in the session's semaphore.  Windows must finish
+        """Run enumerations in the session's semaphore.  Windows must finish
         an enumeration before a new command or enumeration can start
-        '''
+        """
         items = {}
         yield self.init_connection()
         self._session = self.session_manager.get_connection(self.key)
@@ -494,3 +579,85 @@ class EnumerateClient(WinRMClient):
 
         yield self.close_connection()
         returnValue(items)
+
+
+class AssociatorClient(EnumerateClient):
+    """WinRM Client that can return wmi classes that are associated with
+        another wmi class through a single property.
+        First a regular wmi query is run to select objects from a class.
+            e.g. 'select * from Win32_NetworkAdapter'
+        Next we will loop through the results and run the associator query
+        using a specific property of the object as input to return
+        a result class.
+            e.g. for interface in interfaces:
+                "ASSOCIATORS OF {Win32_NetworkAdapter.DeviceID=interface.DeviceID} WHERE ResultClass=Win32_PnPEntity'
+    """
+
+    @inlineCallbacks
+    def associate(self,
+                  seed_class,
+                  associations,
+                  where=None,
+                  resource_uri=DEFAULT_RESOURCE_URI,
+                  fields=['*']):
+        """Method to retrieve associated wmi classes based upon a
+        property from a given class
+
+        seed_class - wmi class which will be initially queried
+        associations - list of dicts containing parameters for
+            the 'ASSOCIATORS of {A}' wql statement.  We dequeue the
+            dicts and can search results from previous wql query to
+            search for nested associations.
+                search_class - initial class to associate with
+                search_property - property on search_class to match
+                return_class - class which will be returned
+                where_type - keyword of association type:
+                    AssocClass = AssocClassName
+                    RequiredAssocQualifier = QualifierName
+                    RequiredQualifier = QualifierName
+                    ResultClass = ClassName
+                    ResultRole = PropertyName
+                    Role = PropertyName
+        where - wql where clause to narrow scope of initial query
+        resource_uri - uri of resource.  this will be the same for both
+            input and result classes.  Limitation of WQL
+        fields - fields to return from seed_class on initial query
+
+        returns dict of seed_class and all return_class results
+            mapped by search_property
+
+        see https://msdn.microsoft.com/en-us/library/aa384793(v=vs.85).aspx
+        """
+        items = {}
+        wql = 'Select {} from {}'.format(','.join(fields), seed_class)
+        if where:
+            wql += ' where {}'.format(where)
+        enum_info = EnumInfo(wql, resource_uri)
+        results = yield self.do_collect([enum_info])
+        input_results = results[enum_info]
+
+        items[seed_class] = input_results
+        while associations:
+            association = associations.pop(0)
+            associate_results = []
+            prop_results = {}
+            for item in input_results:
+                try:
+                    prop = getattr(item, association['search_property'])
+                except AttributeError:
+                    continue
+                else:
+                    wql = "ASSOCIATORS of {{{}.{}='{}'}} WHERE {}={}".format(
+                        association['search_class'],
+                        association['search_property'],
+                        prop,
+                        association['where_type'],
+                        association['return_class'])
+                    enum_info = EnumInfo(wql, resource_uri)
+                    result = yield self.do_collect([enum_info])
+                    associate_results.extend(result[enum_info])
+                    prop_results[prop] = result[enum_info]
+
+            items[association['return_class']] = prop_results
+            input_results = associate_results
+        returnValue(items)
