@@ -24,7 +24,7 @@ from twisted.web.http_headers import Headers
 from twisted.internet.threads import deferToThread
 from . import constants as c
 
-from .krb5 import kinit, ccname, add_trusted_realm
+from .krb5 import kinit, ccname, add_trusted_realm, config
 
 # ZEN-15434 lazy import to avoid segmentation fault during install
 kerberos = None
@@ -132,6 +132,8 @@ def _parse_error_message(xml_str):
         detail = elem.findtext('.//{' + c.XML_NS_SOAP_1_2 + '}Detail/*/*').strip()
     except ParseError:
         return "Malformed XML: {}".format(xml_str)
+    except Exception:
+        return "Unexpected Response ({})".format(xml_str)
     return "{0} {1}".format(text, detail)
 
 
@@ -208,7 +210,9 @@ class AuthGSSClient(object):
         self._conn_info = conn_info
         self._username = conn_info.username
         self._password = conn_info.password
+        self._realm = conn_info.username.split('@')[1].upper()
         self._dcip = conn_info.dcip
+        self._include_dir = conn_info.include_dir
         gssflags = kerberos.GSS_C_CONF_FLAG | kerberos.GSS_C_MUTUAL_FLAG | kerberos.GSS_C_SEQUENCE_FLAG | kerberos.GSS_C_INTEG_FLAG
 
         os.environ['KRB5CCNAME'] = ccname(conn_info.username)
@@ -258,12 +262,19 @@ class AuthGSSClient(object):
                 elif msg == 'Server not found in Kerberos database':
                     raise Exception(msg + ': ' + self._service)
                 log.debug('{0}. Calling kinit.'.format(msg))
-                kinit_result = yield kinit(self._username, self._password, self._dcip)
+                kinit_result = yield kinit(self._username,
+                                           self._password,
+                                           self._dcip,
+                                           includedir=self._include_dir)
                 if kinit_result:
                     # this error is ok.  it just means more
                     # than one process is calling kinit
                     if _KRB_INTERNAL_CACHE_ERR not in kinit_result:
-                        raise Exception(kinit_result)
+                        kinit_result = kinit_result.strip()
+                        extra = ''
+                        if 'Realm not local to KDC while getting initial credentials' in kinit_result:
+                            extra = ' Make sure all KDCs are valid: {}'.format(','.join(config.realms[self._realm]))
+                        raise Exception(kinit_result + extra)
 
         if result_code != kerberos.AUTH_GSS_CONTINUE:
             raise Exception('kerberos authGSSClientStep failed ({0}).'
@@ -417,9 +428,15 @@ class ConnectionInfo(namedtuple(
         'trusted_realm',
         'trusted_kdc',
         'ipaddress',
-        'service'])):
+        'service',
+        'envelope_size',
+        'code_page',
+        'locale',
+        'include_dir'])):
     def __new__(cls, hostname, auth_type, username, password, scheme, port,
-                connectiontype, keytab, dcip, timeout=60, trusted_realm='', trusted_kdc='', ipaddress='', service=''):
+                connectiontype, keytab, dcip, timeout=60, trusted_realm='',
+                trusted_kdc='', ipaddress='', service='', envelope_size=512000,
+                code_page=65001, locale='en-US', include_dir=None):
         if not ipaddress:
             ipaddress = hostname
         if not service:
@@ -428,7 +445,29 @@ class ConnectionInfo(namedtuple(
                                                   username, password, scheme,
                                                   port, connectiontype, keytab,
                                                   dcip, timeout,
-                                                  trusted_realm, trusted_kdc, ipaddress, service)
+                                                  trusted_realm, trusted_kdc,
+                                                  ipaddress, service,
+                                                  envelope_size, code_page, locale,
+                                                  include_dir)
+
+
+def verify_include_dir(conn_info):
+    has_include_dir, include_dir = _has_get_attr(conn_info, 'include_dir')
+    if has_include_dir and include_dir:
+        if not os.path.exists(include_dir):
+            raise Exception("includedir must be a valid location")
+
+
+def verify_code_page(conn_info):
+    has_code_page, code_page = _has_get_attr(conn_info, 'code_page')
+    if not has_code_page or not isinstance(code_page, int):
+        raise Exception("code_page must be an integer")
+
+
+def verify_envelope_size(conn_info):
+    has_envelope_size, envelope_size = _has_get_attr(conn_info, 'envelope_size')
+    if not has_envelope_size or not isinstance(envelope_size, int):
+        raise Exception("envelope_size must be an integer")
 
 
 def verify_hostname(conn_info):
@@ -511,6 +550,7 @@ def verify_conn_info(conn_info):
     verify_port(conn_info)
     verify_connectiontype(conn_info)
     verify_timeout(conn_info)
+    verify_include_dir(conn_info)
 
 
 class RequestSender(object):
@@ -561,6 +601,9 @@ class RequestSender(object):
     def send_request(self, request_template_name, **kwargs):
         log.debug('sending request: {0} {1}'.format(
             request_template_name, kwargs))
+        kwargs['envelope_size'] = getattr(self._conn_info, 'envelope_size', 512000)
+        kwargs['locale'] = getattr(self._conn_info, 'locale', 'en-US')
+        kwargs['code_page'] = getattr(self._conn_info, 'code_page', 65001)
         if not self._url or self._conn_info.auth_type == 'kerberos':
             yield self._set_url_and_headers()
         request = _get_request_template(request_template_name).format(**kwargs)
